@@ -69,7 +69,7 @@ __all__ = [
     'PathOverflowError', 'TooManyChildrenError', 'PathTooDeepError'
 ]
 
-__version__ = (0, 5, 1)
+__version__ = (0, 5, 2)
 __doc__ %= {'version': '.'.join(map(str, __version__))}
 
 
@@ -415,8 +415,10 @@ class MPClassManager(object):
             [opts.path_field, opts.depth_field, opts.tree_id_field],
             opts.pk_field == root_node_id
         ).execute().fetchone()
+        if order_by is None:
+            order_by = opts.pk_field
         self._do_rebuild_subtree(
-            root_node_id, path, depth, tree_id, order_by or opts.pk_field
+            root_node_id, path, depth, tree_id, order_by
         )
 
     def _do_rebuild_subtree(self, root_node_id, root_path, root_depth, \
@@ -857,7 +859,10 @@ def _iter_current_next(sequence):
     iterator = iter(sequence)
     current_item = iterator.next()
     while current_item != _nonexistent:
-        next_item = next(iterator, _nonexistent)
+        try:
+            next_item = iterator.next()
+        except StopIteration:
+            next_item = _nonexistent
         yield (current_item, next_item)
         current_item = next_item
 
@@ -1003,7 +1008,11 @@ class DeclarativeMeta(BaseDeclarativeMeta):
             ]:
             assert isinstance(opts[field], basestring)
             if not hasattr(cls, opts[field]):
-                dct[opts[field]] = sqlalchemy.Column(ftype(), nullable=False)
+                column = sqlalchemy.Column(ftype(), nullable=False)
+                # SQLAlchemy 0.5.x needs this:
+                dct[opts[field]] = column
+                # and SQLAlchemy 0.6/.x needs this:
+                setattr(cls, opts[field], column)
         super(DeclarativeMeta, cls).__init__(name, bases, dct)
         mp_manager = MPManager(cls.__table__, **opts)
         setattr(cls, mp_manager_name, mp_manager)
