@@ -6,6 +6,7 @@ import random
 import unittest
 
 import sqlalchemy
+from sqlalchemy.ext.declarative import declarative_base
 import sqlamp
 
 import tests._testlib as _testlib
@@ -277,7 +278,6 @@ class FunctionalTestCase(_testlib._BaseTestCase):
         self.assertEqual(self.name_pattern, listify(all_trees))
 
     def test_declarative(self):
-        from sqlalchemy.ext.declarative import declarative_base
         BaseNode = declarative_base(metadata=metadata, \
                                     metaclass=sqlamp.DeclarativeMeta)
         class Node(BaseNode):
@@ -292,20 +292,23 @@ class FunctionalTestCase(_testlib._BaseTestCase):
 
         Node.__table__.create()
 
-        root = Node()
-        self.assert_(isinstance(root.MP, sqlamp.MPInstanceManager))
-        self.sess.add(root)
-        self.sess.commit()
-        child = Node()
-        child.parent = root
-        self.sess.add(child)
-        self.sess.commit()
+        try:
+            root = Node()
+            self.assert_(isinstance(root.MP, sqlamp.MPInstanceManager))
+            self.sess.add(root)
+            self.sess.commit()
+            child = Node()
+            child.parent = root
+            self.sess.add(child)
+            self.sess.commit()
 
-        [root, child] = self.sess.query(Node).order_by('id').all()
-        self.assertEqual(root.mp_path, '')
-        self.assertEqual(root.MP_depth, 0)
-        self.assertEqual(child.mp_path, '00000')
-        self.assertEqual(child.MP_depth, 1)
+            [root, child] = self.sess.query(Node).order_by('id').all()
+            self.assertEqual(root.mp_path, '')
+            self.assertEqual(root.MP_depth, 0)
+            self.assertEqual(child.mp_path, '00000')
+            self.assertEqual(child.MP_depth, 1)
+        finally:
+            Node.__table__.delete()
 
     def test_implicit_pk_fk(self):
         tbl = sqlalchemy.Table('tbl2', metadata,
@@ -316,6 +319,97 @@ class FunctionalTestCase(_testlib._BaseTestCase):
         self.assertEqual(tbl.c.id, mpm._mp_opts.pk_field)
         self.assertEqual(tbl.c.pid, mpm._mp_opts.parent_id_field)
 
+    def test_table_inheritance(self):
+        tbl_abstract = sqlalchemy.Table('tbl_abstract', metadata,
+            sqlalchemy.Column('id', sqlalchemy.Integer, primary_key=True),
+            sqlalchemy.Column('parent_id',
+                              sqlalchemy.ForeignKey('tbl_abstract.id')),
+            sqlalchemy.Column('type', sqlalchemy.String(100), nullable=False)
+        )
+        tbl_sub = sqlalchemy.Table('tbl_sub', metadata,
+            sqlalchemy.Column('id', sqlalchemy.ForeignKey('tbl_abstract.id'),
+                              primary_key=True)
+        )
+
+        class AbstractNode(object):
+            mp = sqlamp.MPManager(tbl_abstract)
+        sqlalchemy.orm.mapper(
+            AbstractNode, tbl_abstract, polymorphic_on=tbl_abstract.c.type,
+            polymorphic_identity='abstract',
+            extension=[AbstractNode.mp]
+        )
+
+        class SubNode(AbstractNode):
+            pass
+        sqlalchemy.orm.mapper(SubNode, tbl_sub, inherits=AbstractNode,
+                              polymorphic_identity='sub')
+
+        tbl_abstract.create()
+        tbl_sub.create()
+
+        try:
+            abstract_node = AbstractNode()
+            self.sess.add(abstract_node); self.sess.commit()
+            sub_node = SubNode()
+            sub_node.parent_id = abstract_node.id
+            self.sess.add(sub_node); self.sess.commit()
+            [a, s] = AbstractNode.mp.query_all_trees(self.sess)
+            self.assert_(a is abstract_node)
+            self.assert_(s is sub_node)
+        finally:
+            tbl_abstract.delete()
+            tbl_sub.delete()
+
+    def test_table_inheritance_declarative(self):
+        BaseNode = declarative_base(metadata=metadata, \
+                                    metaclass=sqlamp.DeclarativeMeta)
+        class AbstractNode(BaseNode):
+            __tablename__ = 'node2'
+            __mp_manager__ = 'mp'
+            id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True)
+            parent_id = sqlalchemy.Column(sqlalchemy.ForeignKey('node2.id'))
+            type = sqlalchemy.Column(sqlalchemy.String(100), nullable=False)
+            __mapper_args__ = {'polymorphic_on': type,
+                               'polymorphic_identity': 'abs'}
+        class SubNode(AbstractNode):
+            __tablename__ = 'node2sub'
+            __mapper_args__ = {'polymorphic_identity': 'sub'}
+            id = sqlalchemy.Column(sqlalchemy.ForeignKey('node2.id'),
+                                   primary_key=True)
+        class SubNode2(AbstractNode):
+            __tablename__ = 'node2sub2'
+            __mapper_args__ = {'polymorphic_identity': 'sub2'}
+            id = sqlalchemy.Column(sqlalchemy.ForeignKey('node2.id'),
+                                   primary_key=True)
+
+        AbstractNode.__table__.create()
+        SubNode.__table__.create()
+        SubNode2.__table__.create()
+
+        try:
+            abstract_node = AbstractNode()
+            self.sess.add(abstract_node); self.sess.commit()
+
+            sub_node = SubNode()
+            sub_node.parent_id = abstract_node.id
+            self.sess.add(sub_node); self.sess.commit()
+
+            sub_node2 = SubNode2()
+            sub_node2.parent_id = sub_node.id
+            self.sess.add(sub_node2); self.sess.commit()
+
+            [a, s, s2] = AbstractNode.mp.query_all_trees(self.sess)
+            self.assert_(a is abstract_node)
+            self.assert_(s is sub_node)
+            self.assert_(s2 is sub_node2)
+
+            [s2] = s.mp.query_descendants()
+            self.assert_(s2 is sub_node2)
+        finally:
+            AbstractNode.__table__.delete()
+            SubNode.__table__.delete()
+            SubNode2.__table__.delete()
+
 
 def get_suite():
     return unittest.TestLoader().loadTestsFromTestCase(FunctionalTestCase)
